@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,7 +49,8 @@
 
 package com.openexchange.messaging.generic.internal;
 
-import gnu.trove.TIntArrayList;
+import gnu.trove.list.TIntList;
+import gnu.trove.list.array.TIntArrayList;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
@@ -59,16 +60,15 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import com.openexchange.context.ContextService;
 import com.openexchange.crypto.CryptoException;
 import com.openexchange.crypto.CryptoService;
 import com.openexchange.database.DBPoolingException;
 import com.openexchange.database.DatabaseService;
-import com.openexchange.datatypes.genericonf.DynamicFormDescription;
-import com.openexchange.datatypes.genericonf.FormElement;
-import com.openexchange.datatypes.genericonf.FormElement.Widget;
 import com.openexchange.datatypes.genericonf.storage.GenericConfigStorageException;
 import com.openexchange.datatypes.genericonf.storage.GenericConfigurationStorageService;
+import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.contexts.impl.ContextException;
 import com.openexchange.messaging.MessagingAccount;
@@ -78,7 +78,9 @@ import com.openexchange.messaging.MessagingService;
 import com.openexchange.messaging.generic.DefaultMessagingAccount;
 import com.openexchange.messaging.generic.services.MessagingGenericServiceRegistry;
 import com.openexchange.messaging.registry.MessagingServiceRegistry;
-import com.openexchange.secret.SecretService;
+import com.openexchange.secret.SecretEncryptionFactoryService;
+import com.openexchange.secret.SecretEncryptionService;
+import com.openexchange.secret.SecretEncryptionStrategy;
 import com.openexchange.server.ServiceException;
 import com.openexchange.session.Session;
 import com.openexchange.tools.session.ServerSession;
@@ -86,11 +88,11 @@ import com.openexchange.tools.sql.DBUtils;
 
 /**
  * {@link RdbMessagingAccountStorage} - The messaging account storage backed by database.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  * @since Open-Xchange v6.16
  */
-public class RdbMessagingAccountStorage implements MessagingAccountStorage {
+public class RdbMessagingAccountStorage implements MessagingAccountStorage, SecretEncryptionStrategy<GenericProperty> {
 
     /**
      * The {@link DatabaseService} class.
@@ -106,7 +108,7 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
 
     /**
      * Gets the database-backed instance of account storage manager.
-     * 
+     *
      * @return The database-backed instance of account storage manager
      */
     public static RdbMessagingAccountStorage getInstance() {
@@ -120,10 +122,11 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
         super();
     }
 
-    private static final String SQL_SELECT =
-        "SELECT confId, serviceId, displayName FROM messagingAccount WHERE cid = ? AND user = ? AND serviceId = ? AND account = ?";
+    private static final String SQL_SELECT = "SELECT confId, displayName FROM messagingAccount WHERE cid = ? AND user = ? AND serviceId = ? AND account = ?";
 
-    public MessagingAccount getAccount(final String serviceId, final int id, final Session session) throws MessagingException {
+    private static final String SQL_SELECT_CONFIDS_FOR_USER = "SELECT confId FROM messagingAccount WHERE cid = ? AND user = ? AND serviceId = ?";
+
+    public MessagingAccount getAccount(final String serviceId, final int id, final Session session, final Modifier modifier) throws MessagingException {
         final DatabaseService databaseService = getService(CLAZZ_DB);
         /*
          * Readable connection
@@ -152,24 +155,28 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
                     Integer.valueOf(session.getUserId()),
                     Integer.valueOf(contextId));
             }
+            final MessagingService messagingService = getService(MessagingServiceRegistry.class).getMessagingService(serviceId, session.getUserId(), session.getContextId());
             final DefaultMessagingAccount account = new DefaultMessagingAccount();
-            account.setDisplayName(rs.getString(3));
+            account.setId(id);
+            account.setMessagingService(messagingService);
+            account.setDisplayName(rs.getString(2));
             {
                 final GenericConfigurationStorageService genericConfStorageService = getService(CLAZZ_GEN_CONF);
                 final Map<String, Object> configuration = new HashMap<String, Object>();
-                genericConfStorageService.fill(rc, getContext(session), rs.getInt(1), configuration);
+                final int confId = rs.getInt(1);
+                genericConfStorageService.fill(rc, getContext(session), confId, configuration);
                 /*
                  * Decrypt password fields for clear-text representation in account's configuration
                  */
-                final List<String> passwordElementNames = getPasswordElementNames(serviceId);
-                if (!passwordElementNames.isEmpty()) {
-                    for (final String passwordElementName : passwordElementNames) {
+                final Set<String> secretPropNames = messagingService.getSecretProperties();
+                if (!secretPropNames.isEmpty()) {
+                    for (final String passwordElementName : secretPropNames) {
                         final String toDecrypt = (String) configuration.get(passwordElementName);
                         if (null != toDecrypt) {
                             try {
-                                final String decrypted = decrypt(toDecrypt, session);
+                                final String decrypted = decrypt(toDecrypt, serviceId, id, session, confId, passwordElementName);
                                 configuration.put(passwordElementName, decrypted);
-                            } catch (CryptoException x) {
+                            } catch (final CryptoException x) {
                                 // Must not be fatal
                                 configuration.put(passwordElementName, "");
                                 // Supply a (probably false) password anyway.
@@ -179,15 +186,12 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
                 }
                 account.setConfiguration(configuration);
             }
-            account.setId(id);
-            {
-                final MessagingServiceRegistry registry = getService(MessagingServiceRegistry.class);
-                account.setMessagingService(registry.getMessagingService(rs.getString(2)));
-            }
-            return account;
+            return modifier.modifyOutgoing(account);
         } catch (final SQLException e) {
             throw MessagingExceptionCodes.SQL_ERROR.create(e, e.getMessage());
-        } catch (final GenericConfigStorageException e) {
+        } catch (final MessagingException e) {
+            throw e;
+        } catch (final AbstractOXException e) {
             throw new MessagingException(e);
         } finally {
             DBUtils.closeSQLStuff(rs, stmt);
@@ -195,10 +199,9 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
         }
     }
 
-    private static final String SQL_SELECT_ACCOUNTS =
-        "SELECT account, confId, displayName FROM messagingAccount WHERE cid = ? AND user = ? AND serviceId = ?";
+    private static final String SQL_SELECT_ACCOUNTS = "SELECT account, confId, displayName FROM messagingAccount WHERE cid = ? AND user = ? AND serviceId = ?";
 
-    public List<MessagingAccount> getAccounts(final String serviceId, final Session session) throws MessagingException {
+    public List<MessagingAccount> getAccounts(final String serviceId, final Session session, final Modifier modifier) throws MessagingException {
         final DatabaseService databaseService = getService(CLAZZ_DB);
         /*
          * Readable connection
@@ -226,7 +229,7 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
                 final MessagingService messagingService;
                 {
                     final MessagingServiceRegistry registry = getService(MessagingServiceRegistry.class);
-                    messagingService = registry.getMessagingService(serviceId);
+                    messagingService = registry.getMessagingService(serviceId, session.getUserId(), session.getContextId());
                 }
                 do {
                     final DefaultMessagingAccount account = new DefaultMessagingAccount();
@@ -236,7 +239,7 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
                     account.setConfiguration(configuration);
                     account.setId(rs.getInt(1));
                     account.setMessagingService(messagingService);
-                    accounts.add(account);
+                    accounts.add(modifier.modifyOutgoing(account));
                 } while (rs.next());
             } else {
                 accounts = Collections.emptyList();
@@ -254,13 +257,13 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
 
     /**
      * Gets the identifiers of user-associated accounts of a certain service.
-     * 
+     *
      * @param serviceId The service identifier
      * @param session The session
      * @return The identifiers of user-associated accounts of a certain service
      * @throws MessagingException If identifiers cannot be returned
      */
-    public TIntArrayList getAccountIDs(final String serviceId, final Session session) throws MessagingException {
+    public TIntList getAccountIDs(final String serviceId, final Session session) throws MessagingException {
         final DatabaseService databaseService = getService(CLAZZ_DB);
         /*
          * Readable connection
@@ -272,7 +275,7 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
         } catch (final DBPoolingException e) {
             throw new MessagingException(e);
         }
-        TIntArrayList accounts;
+        TIntList accounts;
         PreparedStatement stmt = null;
         ResultSet rs = null;
         try {
@@ -299,10 +302,9 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
         }
     }
 
-    private static final String SQL_INSERT =
-        "INSERT INTO messagingAccount (cid, user, account, confId, serviceId, displayName) VALUES (?, ?, ?, ?, ?, ?)";
+    private static final String SQL_INSERT = "INSERT INTO messagingAccount (cid, user, account, confId, serviceId, displayName) VALUES (?, ?, ?, ?, ?, ?)";
 
-    public int addAccount(final String serviceId, final MessagingAccount account, final Session session) throws MessagingException {
+    public int addAccount(final String serviceId, final MessagingAccount account, final Session session, final Modifier modifier) throws MessagingException {
         final DatabaseService databaseService = getService(CLAZZ_DB);
         /*
          * Writable connection
@@ -322,6 +324,7 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
             /*
              * Save account configuration using generic conf
              */
+            modifier.modifyIncoming(account);
             final int genericConfId;
             {
                 final GenericConfigurationStorageService genericConfStorageService = getService(CLAZZ_GEN_CONF);
@@ -329,9 +332,10 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
                 /*
                  * Encrypt password fields to not having clear-text representation in database
                  */
-                final List<String> passwordElementNames = getPasswordElementNames(serviceId);
-                if (!passwordElementNames.isEmpty()) {
-                    for (final String passwordElementName : passwordElementNames) {
+                final MessagingService messagingService = getService(MessagingServiceRegistry.class).getMessagingService(serviceId, session.getUserId(), session.getContextId());
+                final Set<String> secretPropNames = messagingService.getSecretProperties();
+                if (!secretPropNames.isEmpty()) {
+                    for (final String passwordElementName : secretPropNames) {
                         final String toCrypt = (String) configuration.get(passwordElementName);
                         if (null != toCrypt) {
                             final String encrypted = encrypt(toCrypt, session);
@@ -371,21 +375,33 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
         }
     }
 
-    private String encrypt(String toCrypt, Session session) throws MessagingException, CryptoException {
-        final CryptoService cryptoService = getService(CryptoService.class);
-        final SecretService secretService = getService(SecretService.class);
-        return cryptoService.encrypt(toCrypt, secretService.getSecret(session));
+    public void update(final String recrypted, final GenericProperty prop) throws AbstractOXException {
+        final HashMap<String, Object> update = new HashMap<String, Object>();
+        update.put(prop.propertyName, recrypted);
+        final GenericConfigurationStorageService genericConfStorageService = getService(CLAZZ_GEN_CONF);
+        final Session session = prop.session;
+        genericConfStorageService.update(getContext(session), prop.confId, update);
+        // Invalidate account
+        try {
+            CachingMessagingAccountStorage.getInstance().invalidate(prop.serviceId, prop.id, session.getUserId(), session.getContextId());
+        } catch (final Exception e) {
+            // Ignore
+        }
     }
-    
-    private String decrypt(String toDecrypt, Session session) throws MessagingException, CryptoException {
-        final CryptoService cryptoService = getService(CryptoService.class);
-        final SecretService secretService = getService(SecretService.class);
-        return cryptoService.decrypt(toDecrypt, secretService.getSecret(session));
+
+    private String encrypt(final String toCrypt, final Session session) throws MessagingException, CryptoException {
+        final SecretEncryptionService<GenericProperty> encryptionService = getService(SecretEncryptionFactoryService.class).createService(this);
+        return encryptionService.encrypt(session, toCrypt);
+    }
+
+    private String decrypt(final String toDecrypt, final String serviceId, final int id, final Session session, final int confId, final String propertyName) throws AbstractOXException {
+        final SecretEncryptionService<GenericProperty> encryptionService = getService(SecretEncryptionFactoryService.class).createService(this);
+        return encryptionService.decrypt(session, toDecrypt, new GenericProperty(confId, propertyName, serviceId, id, session));
     }
 
     private static final String SQL_DELETE = "DELETE FROM messagingAccount WHERE cid = ? AND user = ? AND serviceId = ? AND account = ?";
 
-    public void deleteAccount(final String serviceId, final MessagingAccount account, final Session session) throws MessagingException {
+    public void deleteAccount(final String serviceId, final MessagingAccount account, final Session session, final Modifier modifier) throws MessagingException {
         final DatabaseService databaseService = getService(CLAZZ_DB);
         /*
          * Writable connection
@@ -402,6 +418,7 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
         }
         PreparedStatement stmt = null;
         try {
+            modifier.modifyIncoming(account);
             final int accountId = account.getId();
             /*
              * Delete account configuration using generic conf
@@ -438,10 +455,9 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
         }
     }
 
-    private static final String SQL_UPDATE =
-        "UPDATE messagingAccount SET displayName = ? WHERE cid = ? AND user = ? AND serviceId = ? AND account = ?";
+    private static final String SQL_UPDATE = "UPDATE messagingAccount SET displayName = ? WHERE cid = ? AND user = ? AND serviceId = ? AND account = ?";
 
-    public void updateAccount(final String serviceId, final MessagingAccount account, final Session session) throws MessagingException {
+    public void updateAccount(final String serviceId, final MessagingAccount account, final Session session, final Modifier modifier) throws MessagingException {
         final DatabaseService databaseService = getService(CLAZZ_DB);
         /*
          * Writable connection
@@ -458,6 +474,7 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
         }
         PreparedStatement stmt = null;
         try {
+            modifier.modifyIncoming(account);
             /*
              * Update account configuration using generic conf
              */
@@ -470,9 +487,10 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
                     /*
                      * Encrypt password fields to not having clear-text representation in database
                      */
-                    final List<String> passwordElementNames = getPasswordElementNames(serviceId);
-                    if (!passwordElementNames.isEmpty()) {
-                        for (final String passwordElementName : passwordElementNames) {
+                    final MessagingService messagingService = getService(MessagingServiceRegistry.class).getMessagingService(serviceId, session.getUserId(), session.getContextId());
+                    final Set<String> secretPropNames = messagingService.getSecretProperties();
+                    if (!secretPropNames.isEmpty()) {
+                        for (final String passwordElementName : secretPropNames) {
                             final String toCrypt = (String) configuration.get(passwordElementName);
                             if (null != toCrypt) {
                                 final String encrypted = encrypt(toCrypt, session);
@@ -557,24 +575,145 @@ public class RdbMessagingAccountStorage implements MessagingAccountStorage {
         }
     }
 
-    /**
-     * Gets the names of those {@link FormElement} associated with given identifier's messaging service which indicate to be of type
-     * {@link Widget#PASSWORD password}.
-     * 
-     * @param serviceId The service identifier
-     * @return The password field names
-     * @throws MessagingException If names cannot be returned
-     */
-    private static List<String> getPasswordElementNames(final String serviceId) throws MessagingException {
-        final DynamicFormDescription formDescription =
-            getService(MessagingServiceRegistry.class).getMessagingService(serviceId).getFormDescription();
-        final List<String> retval = new ArrayList<String>(2);
-        for (final FormElement formElement : formDescription) {
-            if (Widget.PASSWORD.equals(formElement.getWidget())) {
-                retval.add(formElement.getName());
+    public String checkSecretCanDecryptStrings(final MessagingService parentService, final Session session, final String secret) throws MessagingException {
+        final Set<String> secretProperties = parentService.getSecretProperties();
+        if (secretProperties.isEmpty()) {
+            return null;
+        }
+        final TIntList confIds = getConfIDsForUser(session.getContextId(), session.getUserId(), parentService.getId());
+        final GenericConfigurationStorageService genericConfStorageService = getService(CLAZZ_GEN_CONF);
+        final CryptoService cryptoService = getService(CryptoService.class);
+
+        final Context ctx = getContext(session);
+        final HashMap<String, Object> content = new HashMap<String, Object>();
+        int confId = -1;
+        try {
+            for (int i = 0, size = confIds.size(); i < size; i++) {
+                confId = confIds.get(i);
+                content.clear();
+                genericConfStorageService.fill(ctx, confId, content);
+
+                for (final String field : secretProperties) {
+                    final String encrypted = (String) content.get(field);
+                    if (encrypted != null) {
+                        cryptoService.decrypt(encrypted, secret);
+                    }
+                }
+            }
+        } catch (final GenericConfigStorageException e) {
+            throw new MessagingException(e);
+        } catch (final CryptoException e) {
+            return "Messaging dynamic configuration with id "+confId+" could not be decrypted";
+        }
+        return null;
+    }
+
+    private TIntList getConfIDsForUser(final int contextId, final int userId, final String serviceId) throws MessagingException {
+        final TIntList confIds = new TIntArrayList(20);
+        final DatabaseService databaseService = getService(CLAZZ_DB);
+        PreparedStatement stmt = null;
+        ResultSet rs = null;
+        Connection con = null;
+        try {
+            con = databaseService.getReadOnly(contextId);
+            stmt = con.prepareStatement(SQL_SELECT_CONFIDS_FOR_USER);
+            int pos = 1;
+            stmt.setInt(pos++, contextId);
+            stmt.setInt(pos++, userId);
+            stmt.setString(pos++, serviceId);
+
+            rs = stmt.executeQuery();
+
+            while (rs.next()) {
+                final int confId = rs.getInt(1);
+                confIds.add(confId);
+            }
+
+        } catch (final SQLException e) {
+            throw MessagingExceptionCodes.SQL_ERROR.create(e, e.getMessage());
+        } catch (final DBPoolingException e) {
+            throw new MessagingException(e);
+        } finally {
+            DBUtils.closeSQLStuff(rs, stmt);
+            if (con != null) {
+                databaseService.backReadOnly(contextId, con);
+            }
+        }
+        return confIds;
+    }
+
+    public void migrateToNewSecret(final MessagingService parentService, final String oldSecret, final String newSecret, final Session session) throws MessagingException {
+        final Set<String> secretProperties = parentService.getSecretProperties();
+        if (secretProperties.isEmpty()) {
+            return;
+        }
+        final TIntList confIds = getConfIDsForUser(session.getContextId(), session.getUserId(), parentService.getId());
+        final GenericConfigurationStorageService genericConfStorageService = getService(CLAZZ_GEN_CONF);
+        final CryptoService cryptoService = getService(CryptoService.class);
+
+        final Context ctx = getContext(session);
+        final HashMap<String, Object> content = new HashMap<String, Object>();
+        try {
+            for (int i = 0, size = confIds.size(); i < size; i++) {
+                final int confId = confIds.get(i);
+                content.clear();
+                genericConfStorageService.fill(ctx, confId, content);
+                final HashMap<String, Object> update = new HashMap<String, Object>();
+                for (final String field : secretProperties) {
+                    final String encrypted = (String) content.get(field);
+                    if (encrypted != null) {
+                        try {
+                            // Try using the new secret. Maybe this account doesn't need the migration
+                            cryptoService.decrypt(encrypted, newSecret);
+                        } catch (final CryptoException x) {
+                            // Needs migration
+                            final String transcripted = cryptoService.encrypt(cryptoService.decrypt(encrypted, oldSecret), newSecret);
+                            update.put(field, transcripted);
+                        }
+                    }
+                }
+                genericConfStorageService.update(ctx, confId, update);
+            }
+        } catch (final GenericConfigStorageException e) {
+            throw new MessagingException(e);
+        } catch (final CryptoException e) {
+            throw new MessagingException(e);
+        }
+    }
+
+    private static final String ACCOUNT_EXISTS = "SELECT 1 FROM messagingAccount WHERE cid = ? AND user = ? LIMIT 1";
+    
+    public boolean hasAccount(final MessagingService parentService, final Session session) throws MessagingException {
+        final Set<String> secretProperties = parentService.getSecretProperties();
+        if (secretProperties.isEmpty()) {
+            return false;
+        }
+        
+        final DatabaseService databaseService = getService(CLAZZ_DB);
+        PreparedStatement stmt = null;
+        ResultSet rs = null;
+        Connection con = null;
+        try {
+            con = databaseService.getReadOnly(session.getContextId());
+            stmt = con.prepareStatement(ACCOUNT_EXISTS);
+            int pos = 1;
+            stmt.setInt(pos++, session.getContextId());
+            stmt.setInt(pos++, session.getUserId());
+
+            rs = stmt.executeQuery();
+            
+            return rs.next();
+
+        } catch (final SQLException e) {
+            throw MessagingExceptionCodes.SQL_ERROR.create(e, e.getMessage());
+        } catch (final DBPoolingException e) {
+            throw new MessagingException(e);
+        } finally {
+            DBUtils.closeSQLStuff(rs, stmt);
+            if (con != null) {
+                databaseService.backReadOnly(session.getContextId(), con);
             }
         }
-        return retval;
     }
 
 }
