@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -71,6 +71,7 @@ import com.openexchange.mail.mime.HeaderName;
 import com.openexchange.mail.mime.datasource.StreamDataSource;
 import com.openexchange.messaging.BinaryContent;
 import com.openexchange.messaging.ContentType;
+import com.openexchange.messaging.ManagedFileContent;
 import com.openexchange.messaging.MessagingContent;
 import com.openexchange.messaging.MessagingException;
 import com.openexchange.messaging.MessagingExceptionCodes;
@@ -81,13 +82,13 @@ import com.openexchange.messaging.StringContent;
 
 /**
  * {@link MimeMessagingPart} - The MIME messaging part.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  * @since Open-Xchange v6.16
  */
 public class MimeMessagingPart implements MessagingPart {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(MimeMessagingPart.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MimeMessagingPart.class));
 
     private static final boolean DEBUG = LOG.isDebugEnabled();
 
@@ -144,6 +145,48 @@ public class MimeMessagingPart implements MessagingPart {
 
     } // End of BinaryContentISP class
 
+    /**
+     * An {@link StreamDataSource.InputStreamProvider} backed by a {@link BinaryContent}.
+     */
+    private static final class ManagedFileContentISP implements StreamDataSource.InputStreamProvider {
+
+        private final ManagedFileContent fileContent;
+
+        ManagedFileContentISP(final ManagedFileContent fileContent) {
+            super();
+            this.fileContent = fileContent;
+        }
+
+        public InputStream getInputStream() throws IOException {
+            try {
+                return fileContent.getData();
+            } catch (final MessagingException e) {
+                final Throwable cause = e.getCause();
+                if (cause instanceof IOException) {
+                    throw (IOException) cause;
+                }
+                if (cause instanceof javax.mail.MessagingException) {
+                    final javax.mail.MessagingException me = (javax.mail.MessagingException) cause;
+                    final Exception nextException = me.getNextException();
+                    if (nextException instanceof IOException) {
+                        throw (IOException) nextException;
+                    }
+                    final IOException ioException = new IOException(me.getMessage());
+                    ioException.initCause(me);
+                    throw ioException;
+                }
+                final IOException ioException = new IOException(e.getMessage());
+                ioException.initCause(e);
+                throw ioException;
+            }
+        }
+
+        public String getName() {
+            return null;
+        }
+
+    } // End of BinaryContentISP class
+
     /**
      * Adds an appropriate {@link MessagingHeader} to a collection.
      */
@@ -151,7 +194,7 @@ public class MimeMessagingPart implements MessagingPart {
 
         /**
          * Adds an appropriate {@link MessagingHeader} created from given header to specified collection
-         * 
+         *
          * @param header The header to convert to a {@link MessagingHeader} instance
          * @param collection The collection to add to
          * @throws MessagingException If adding header fails
@@ -290,7 +333,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Initializes a new {@link MimeMessagingPart}.
-     * 
+     *
      * @param part The part
      */
     public MimeMessagingPart(final MimePart part) {
@@ -473,7 +516,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Sets the section identifier.
-     * 
+     *
      * @param sectionId The section identifier
      */
     public void setSectionId(final String sectionId) {
@@ -496,7 +539,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Sets this part's size. If passed argument is less than or equal to zero, specified size will not be set.
-     * 
+     *
      * @param size The size to set
      */
     public void setSize(final long size) {
@@ -513,7 +556,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Adds given header collection to the existing headers of this messaging part.
-     * 
+     *
      * @param headers The headers to add
      * @throws MessagingException If adding headers fails
      */
@@ -533,7 +576,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Adds specified header value to the existing values for the associated header name.
-     * 
+     *
      * @param headerName The header name
      * @param headerValue The header value
      * @throws MessagingException If adding header fails
@@ -553,7 +596,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Adds specified header value to the existing values for the associated header name.
-     * 
+     *
      * @param header The header to add
      * @throws MessagingException If adding header fails
      */
@@ -572,7 +615,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Remove all headers associated with specified name.
-     * 
+     *
      * @param headerName The header name
      * @throws MessagingException If header removal fails
      */
@@ -591,7 +634,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Sets the given multipart as this part's content.
-     * 
+     *
      * @param mp The multipart
      * @throws MessagingException If multipart cannot be set as content
      */
@@ -612,7 +655,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Sets this part's content.
-     * 
+     *
      * @param content The content
      * @param type The content type
      * @throws MessagingException If content cannot be applied
@@ -628,6 +671,8 @@ public class MimeMessagingPart implements MessagingPart {
             } else if (content instanceof SimpleContent<?>) {
                 if (content instanceof BinaryContent) {
                     part.setDataHandler(new DataHandler(new StreamDataSource(new BinaryContentISP((BinaryContent) content), type)));
+                } else if (content instanceof ManagedFileContent) {
+                    part.setDataHandler(new DataHandler(new StreamDataSource(new ManagedFileContentISP((ManagedFileContent) content), type)));
                 } else if (content instanceof StringContent) {
                     final MimeContentType mct = new MimeContentType(type);
                     part.setText(((StringContent) content).getData().toString(), mct.getCharsetParameter(), mct.getSubType());
@@ -653,7 +698,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Set the disposition of this part.
-     * 
+     *
      * @param disposition The disposition to set
      * @throws MessagingException If setting disposition fails
      */
@@ -670,7 +715,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Sets this part's file name.
-     * 
+     *
      * @param filename The file name
      * @throws MessagingException If setting file name fails
      */
@@ -687,7 +732,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Sets given header collection to this messaging part.
-     * 
+     *
      * @param headers The headers to set
      * @throws MessagingException If setting headers fails
      */
@@ -719,7 +764,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Set the value for this header name. Replaces all existing header values associated with header name.
-     * 
+     *
      * @param headerName The header name
      * @param headerValue The header value
      * @throws MessagingException If setting header fails
@@ -739,7 +784,7 @@ public class MimeMessagingPart implements MessagingPart {
 
     /**
      * Set the value for this header name. Replaces all existing header values associated with header name.
-     * 
+     *
      * @param header The header to set
      * @throws MessagingException If setting header fails
      */
@@ -765,7 +810,7 @@ public class MimeMessagingPart implements MessagingPart {
      * determine what charset to use.
      * <p>
      * If the charset is already known, use the <code>setText</code> method that takes the charset parameter.
-     * 
+     *
      * @param text The text content to set
      * @throws MessagingException If text cannot be applied
      * @see #setText(String text, String charset)
@@ -777,7 +822,7 @@ public class MimeMessagingPart implements MessagingPart {
     /**
      * Convenience method that sets the given String as this part's content, with a MIME type of "text/plain" and the specified charset. The
      * given Unicode string will be charset-encoded using the specified charset. The charset is also used to set the "charset" parameter.
-     * 
+     *
      * @param text The text content to set
      * @param charset The charset to use for the text
      * @throws MessagingException If text cannot be applied
@@ -790,7 +835,7 @@ public class MimeMessagingPart implements MessagingPart {
      * Convenience method that sets the given String as this part's content, with a primary MIME type of "text" and the specified MIME
      * subtype. The given Unicode string will be charset-encoded using the specified charset. The charset is also used to set the "charset"
      * parameter.
-     * 
+     *
      * @param text The text content to set
      * @param charset The charset to use for the text
      * @param subtype The MIME subtype to use (e.g., "html")
