@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -54,6 +54,7 @@ import java.io.ByteArrayInputStream;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.List;
 import org.osgi.framework.ServiceRegistration;
 import org.osgi.util.tracker.ServiceTracker;
@@ -66,18 +67,22 @@ import com.openexchange.datatypes.genericonf.storage.GenericConfigurationStorage
 import com.openexchange.groupware.delete.DeleteListener;
 import com.openexchange.groupware.update.UpdateTask;
 import com.openexchange.groupware.update.UpdateTaskProviderService;
+import com.openexchange.messaging.MessagingService;
 import com.openexchange.messaging.generic.groupware.MessagingGenericCreateTableTask;
 import com.openexchange.messaging.generic.groupware.MessagingGenericDeleteListener;
 import com.openexchange.messaging.generic.internal.CachingMessagingAccountStorage;
+import com.openexchange.messaging.generic.secret.MessagingSecretHandling;
 import com.openexchange.messaging.registry.MessagingServiceRegistry;
 import com.openexchange.secret.SecretService;
 import com.openexchange.secret.osgi.tools.WhiteboardSecretService;
+import com.openexchange.secret.recovery.EncryptedItemDetectorService;
+import com.openexchange.secret.recovery.SecretMigrator;
 import com.openexchange.server.osgiservice.DeferredActivator;
 import com.openexchange.server.osgiservice.ServiceRegistry;
 
 /**
  * {@link MessagingGenericActivator}
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  * @since Open-Xchange v6.16
  */
@@ -102,7 +107,7 @@ public class MessagingGenericActivator extends DeferredActivator {
 
     @Override
     protected void handleAvailability(final Class<?> clazz) {
-        final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(MessagingGenericActivator.class);
+        final org.apache.commons.logging.Log logger = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MessagingGenericActivator.class));
         if (logger.isInfoEnabled()) {
             logger.info("Re-available service: " + clazz.getName());
         }
@@ -111,7 +116,7 @@ public class MessagingGenericActivator extends DeferredActivator {
 
     @Override
     protected void handleUnavailability(final Class<?> clazz) {
-        final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(MessagingGenericActivator.class);
+        final org.apache.commons.logging.Log logger = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MessagingGenericActivator.class));
         if (logger.isWarnEnabled()) {
             logger.warn("Absent service: " + clazz.getName());
         }
@@ -128,39 +133,43 @@ public class MessagingGenericActivator extends DeferredActivator {
                 final ServiceRegistry registry = getServiceRegistry();
                 registry.clearRegistry();
                 final Class<?>[] classes = getNeededServices();
-                for (int i = 0; i < classes.length; i++) {
-                    final Object service = getService(classes[i]);
+                for (final Class<?> classe : classes) {
+                    final Object service = getService(classe);
                     if (null != service) {
-                        registry.addService(classes[i], service);
+                        registry.addService(classe, service);
                     }
                 }
                 registry.addService(SecretService.class, secretService = new WhiteboardSecretService(context));
                 secretService.open();
             }
-            
+
             {
                 /*
                  * MessagingAccount region with 5 minutes time-out
                  */
                 final String regionName = CachingMessagingAccountStorage.getRegionName();
-                final byte[] ccf = ("jcs.region."+regionName+"=LTCP\n" + 
-                		"jcs.region."+regionName+".cacheattributes=org.apache.jcs.engine.CompositeCacheAttributes\n" + 
-                		"jcs.region."+regionName+".cacheattributes.MaxObjects=10000000\n" + 
-                		"jcs.region."+regionName+".cacheattributes.MemoryCacheName=org.apache.jcs.engine.memory.lru.LRUMemoryCache\n" + 
-                		"jcs.region."+regionName+".cacheattributes.UseMemoryShrinker=true\n" + 
-                		"jcs.region."+regionName+".cacheattributes.MaxMemoryIdleTimeSeconds=180\n" + 
-                		"jcs.region."+regionName+".cacheattributes.ShrinkerIntervalSeconds=60\n" + 
-                		"jcs.region."+regionName+".elementattributes=org.apache.jcs.engine.ElementAttributes\n" + 
-                		"jcs.region."+regionName+".elementattributes.IsEternal=false\n" + 
-                		"jcs.region."+regionName+".elementattributes.MaxLifeSeconds=300\n" + 
-                		"jcs.region."+regionName+".elementattributes.IdleTime=180\n" + 
-                		"jcs.region."+regionName+".elementattributes.IsSpool=false\n" + 
-                		"jcs.region."+regionName+".elementattributes.IsRemote=false\n" + 
+                final byte[] ccf = ("jcs.region."+regionName+"=LTCP\n" +
+                		"jcs.region."+regionName+".cacheattributes=org.apache.jcs.engine.CompositeCacheAttributes\n" +
+                		"jcs.region."+regionName+".cacheattributes.MaxObjects=10000000\n" +
+                		"jcs.region."+regionName+".cacheattributes.MemoryCacheName=org.apache.jcs.engine.memory.lru.LRUMemoryCache\n" +
+                		"jcs.region."+regionName+".cacheattributes.UseMemoryShrinker=true\n" +
+                		"jcs.region."+regionName+".cacheattributes.MaxMemoryIdleTimeSeconds=180\n" +
+                		"jcs.region."+regionName+".cacheattributes.ShrinkerIntervalSeconds=60\n" +
+                		"jcs.region."+regionName+".elementattributes=org.apache.jcs.engine.ElementAttributes\n" +
+                		"jcs.region."+regionName+".elementattributes.IsEternal=false\n" +
+                		"jcs.region."+regionName+".elementattributes.MaxLifeSeconds=300\n" +
+                		"jcs.region."+regionName+".elementattributes.IdleTime=180\n" +
+                		"jcs.region."+regionName+".elementattributes.IsSpool=false\n" +
+                		"jcs.region."+regionName+".elementattributes.IsRemote=false\n" +
                 		"jcs.region."+regionName+".elementattributes.IsLateral=false\n").getBytes();
                 getService(CacheService.class).loadConfiguration(new ByteArrayInputStream(ccf));
             }
 
             trackers = new ArrayList<ServiceTracker>();
+
+            final ServiceTracker messagingServiceTracker = new ServiceTracker(context, MessagingService.class.getName(), null);
+            trackers.add(messagingServiceTracker);
+
             for (final ServiceTracker tracker : trackers) {
                 tracker.open();
             }
@@ -174,9 +183,35 @@ public class MessagingGenericActivator extends DeferredActivator {
             }, null));
             registrations.add(context.registerService(CreateTableService.class.getName(), createTableTask, null));
             registrations.add(context.registerService(DeleteListener.class.getName(), new MessagingGenericDeleteListener(), null));
-            
+
+
+            // Secret Handling
+
+            {
+                final MessagingSecretHandling secretHandling = new MessagingSecretHandling() {
+                    @Override
+                    protected Collection<MessagingService> getMessagingServices() {
+                        final Object[] objects = messagingServiceTracker.getServices();
+                        if(objects == null){
+                            return Collections.emptyList();
+                        }
+                        final List<MessagingService> list = new ArrayList<MessagingService>(objects.length);
+                        for (final Object o : objects) {
+                            list.add((MessagingService) o);
+                        }
+
+                        return list;
+                    }
+                };
+
+
+                registrations.add(context.registerService(EncryptedItemDetectorService.class.getName(), secretHandling, null));
+                registrations.add(context.registerService(SecretMigrator.class.getName(), secretHandling, null));
+
+            }
+
         } catch (final Exception e) {
-            org.apache.commons.logging.LogFactory.getLog(MessagingGenericActivator.class).error(e.getMessage(), e);
+            com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MessagingGenericActivator.class)).error(e.getMessage(), e);
             throw e;
         }
     }
@@ -184,17 +219,19 @@ public class MessagingGenericActivator extends DeferredActivator {
     @Override
     protected void stopBundle() throws Exception {
         try {
+            final List<ServiceTracker> trackers = this.trackers;
             if (null != trackers) {
                 while (!trackers.isEmpty()) {
                     trackers.remove(0).close();
                 }
-                trackers = null;
+                this.trackers = null;
             }
+            final List<ServiceRegistration> registrations = this.registrations;
             if (null != registrations) {
                 while (!registrations.isEmpty()) {
                     registrations.remove(0).unregister();
                 }
-                registrations = null;
+                this.registrations = null;
             }
             final CacheService cacheService = getService(CacheService.class);
             if (null != cacheService) {
@@ -204,9 +241,13 @@ public class MessagingGenericActivator extends DeferredActivator {
              * Clear service registry
              */
             getServiceRegistry().clearRegistry();
-            secretService.close();
+            final WhiteboardSecretService secretService = this.secretService;
+            if (null != secretService) {
+                secretService.close();
+                this.secretService = null;
+            }
         } catch (final Exception e) {
-            org.apache.commons.logging.LogFactory.getLog(MessagingGenericActivator.class).error(e.getMessage(), e);
+            com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MessagingGenericActivator.class)).error(e.getMessage(), e);
             throw e;
         }
     }
